@@ -14,14 +14,18 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #define FLEET_H_
 
 #include "Personality.h"
+#include "Sale.h"
 
 #include <list>
 #include <memory>
+#include <set>
 #include <string>
+#include <utility>
 #include <vector>
 
 class DataNode;
 class Government;
+class Outfit;
 class Phrase;
 class Planet;
 class Ship;
@@ -37,17 +41,27 @@ class System;
 // names are chosen based on a given random "phrase" generator.
 class Fleet {
 public:
+	Fleet() = default;
+	// Construct and Load() at the same time.
+	Fleet(const DataNode &node);
+	
 	void Load(const DataNode &node);
 	
+	// Determine if this fleet template uses well-defined data.
+	bool IsValid(bool requireGovernment = true) const;
+	
 	// Get the government of this fleet.
 	const Government *GetGovernment() const;
 	
+	// Choose a fleet to be created during flight, and have it enter the system via jump or planetary departure.
 	void Enter(const System &system, std::list<std::shared_ptr<Ship>> &ships, const Planet *planet = nullptr) const;
-	// Place a fleet in the given system, already "in action."
+	// Place a fleet in the given system, already "in action." If the carried flag is set, only
+	// uncarried ships will be added to the list (as any carriables will be stored in bays).
 	void Place(const System &system, std::list<std::shared_ptr<Ship>> &ships, bool carried = true) const;
 	
 	// Do the randomization to make a ship enter or be in the given system.
-	static void Enter(const System &system, Ship &ship);
+	// Return the system that was chosen for the ship to enter from.
+	static const System *Enter(const System &system, Ship &ship, const System *source = nullptr);
 	static void Place(const System &system, Ship &ship);
 	
 	int64_t Strength() const;
@@ -65,7 +79,7 @@ private:
 	
 private:
 	const Variant &ChooseVariant() const;
-	static Point ChooseCenter(const System &system);
+	static std::pair<Point, double> ChooseCenter(const System &system);
 	std::vector<std::shared_ptr<Ship>> Instantiate(const Variant &variant) const;
 	bool PlaceFighter(std::shared_ptr<Ship> fighter, std::vector<std::shared_ptr<Ship>> &placed) const;
 	void SetCargo(Ship *ship) const;
@@ -77,9 +91,12 @@ private:
 	const Phrase *names = nullptr;
 	const Phrase *fighterNames = nullptr;
 	std::vector<Variant> variants;
+	// The sum of all available variant weights.
+	int total = 0;
+	// The number of different items the ships in this fleet will carry in cargo.
 	int cargo = 3;
 	std::vector<std::string> commodities;
-	int total = 0;
+	std::set<const Sale<Outfit> *> outfitters;
 	
 	Personality personality;
 };
