@@ -12,13 +12,19 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include "MapDetailPanel.h"
 
+#include "text/alignment.hpp"
+#include "Angle.h"
 #include "Color.h"
 #include "Command.h"
-#include "Font.h"
-#include "FontSet.h"
-#include "Format.h"
+#include "CoreStartData.h"
+#include "Dialog.h"
+#include "text/DisplayText.h"
+#include "text/Font.h"
+#include "text/FontSet.h"
+#include "text/Format.h"
 #include "GameData.h"
 #include "Government.h"
+#include "text/layout.hpp"
 #include "MapOutfitterPanel.h"
 #include "MapShipyardPanel.h"
 #include "pi.h"
@@ -26,6 +32,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "PlayerInfo.h"
 #include "PointerShader.h"
 #include "Politics.h"
+#include "Preferences.h"
 #include "Radar.h"
 #include "RingShader.h"
 #include "Screen.h"
@@ -36,11 +43,13 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "StellarObject.h"
 #include "System.h"
 #include "Trade.h"
+#include "text/truncate.hpp"
 #include "UI.h"
-#include "WrappedText.h"
+#include "text/WrappedText.h"
 
 #include <algorithm>
 #include <cmath>
+#include <limits>
 #include <set>
 #include <utility>
 #include <vector>
@@ -96,6 +105,8 @@ void MapDetailPanel::Step()
 	MapPanel::Step();
 	if(!player.GetPlanet())
 		DoHelp("map");
+	if(GetUI()->IsTop(this) && player.GetPlanet() && player.GetDate() >= player.StartData().GetDate() + 12)
+		DoHelp("map advanced ports");
 }
 
 
@@ -139,7 +150,7 @@ bool MapDetailPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command
 		// Depending on whether the flagship has a jump drive, the possible links
 		// we can travel along are different:
 		bool hasJumpDrive = player.Flagship()->Attributes().Get("jump drive");
-		const set<const System *> &links = hasJumpDrive ? source->Neighbors() : source->Links();
+		const set<const System *> &links = hasJumpDrive ? source->JumpNeighbors(player.Flagship()->JumpRange()) : source->Links();
 		
 		// For each link we can travel from this system, check whether the link
 		// is closer to the current angle (while still being larger) than any
@@ -152,9 +163,9 @@ bool MapDetailPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command
 			// player has not visited either end of them.
 			if(it == original)
 				continue;
-			if(!player.HasSeen(it))
+			if(!player.HasSeen(*it))
 				continue;
-			if(!(hasJumpDrive || player.HasVisited(it) || player.HasVisited(source)))
+			if(!(hasJumpDrive || player.HasVisited(*it) || player.HasVisited(*source)))
 				continue;
 			
 			// Generate a sortable angle with vector length as a tiebreaker.
@@ -205,17 +216,23 @@ bool MapDetailPanel::Click(int x, int y, int clicks)
 {
 	if(x < Screen::Left() + 160)
 	{
+		// The player clicked in the left-hand interface. This could be the system
+		// name, the system government, a planet box, the commodity listing, or nothing.
 		if(y >= tradeY && y < tradeY + 200)
 		{
+			// The player clicked on a tradable commodity. Color the map by its price.
 			SetCommodity((y - tradeY) / 20);
 			return true;
 		}
+		// Clicking the system name activates the view of the player's reputation with various governments.
 		else if(y < governmentY)
 			SetCommodity(SHOW_REPUTATION);
+		// Clicking the government name activates the view of system / planet ownership.
 		else if(y >= governmentY && y < governmentY + 20)
 			SetCommodity(SHOW_GOVERNMENT);
 		else
 		{
+			// The player clicked within the region associated with this system's planets.
 			for(const auto &it : planetY)
 				if(y >= it.second && y < it.second + 110)
 				{
@@ -228,6 +245,7 @@ bool MapDetailPanel::Click(int x, int y, int clicks)
 							SHOW_REPUTATION, SHOW_SHIPYARD, SHOW_OUTFITTER, SHOW_VISITED};
 						SetCommodity(SHOW[row]);
 						
+						// Double-click the Shipyard or Outfitter line to open that map view.
 						if(clicks > 1 && SHOW[row] == SHOW_SHIPYARD)
 						{
 							GetUI()->Pop(this);
@@ -245,6 +263,8 @@ bool MapDetailPanel::Click(int x, int y, int clicks)
 	}
 	else if(x >= Screen::Right() - 240 && y <= Screen::Top() + 270)
 	{
+		// The player has clicked within the "orbits" scene.
+		// Select the nearest planet to the click point.
 		Point click = Point(x, y);
 		selectedPlanet = nullptr;
 		double distance = numeric_limits<double>::infinity();
@@ -262,18 +282,10 @@ bool MapDetailPanel::Click(int x, int y, int clicks)
 		
 		return true;
 	}
-	else if(y >= Screen::Bottom() - 40 && x >= Screen::Right() - 335 && x < Screen::Right() - 265)
-	{
-		// The user clicked the "done" button.
-		return DoKey(SDLK_d);
-	}
-	else if(y >= Screen::Bottom() - 40 && x >= Screen::Right() - 415 && x < Screen::Right() - 345)
-	{
-		// The user clicked the "missions" button.
-		return DoKey(SDLK_PAGEDOWN);
-	}
 	
+	// The click was not on an interface element, so check if it was on a system.
 	MapPanel::Click(x, y, clicks);
+	// If the system just changed, the selected planet is no longer valid.
 	if(selectedPlanet && !selectedPlanet->IsInSystem(selectedSystem))
 		selectedPlanet = nullptr;
 	return true;
@@ -281,6 +293,35 @@ bool MapDetailPanel::Click(int x, int y, int clicks)
 
 
 
+bool MapDetailPanel::RClick(int x, int y)
+{
+	if(!Preferences::Has("System map sends move orders"))
+		return true;
+	// TODO: rewrite the map panels to be driven from interfaces.txt so these XY
+	// positions aren't hard-coded.
+	else if(x >= Screen::Right() - 240 && y >= Screen::Top() + 10 && y <= Screen::Top() + 270)
+	{
+		// Only handle clicks on the actual orbits element, rather than the whole UI region.
+		// (Note: this isn't perfect, and the clickable area extends into the angled sides a bit.)
+		const Point orbitCenter(Screen::TopRight() + Point(-120., 160.));
+		auto uiClick = Point(x, y) - orbitCenter;
+		if(uiClick.Length() > 130)
+			return true;
+		
+		// Only issue movement orders if the player is in-flight.
+		if(player.GetPlanet())
+			GetUI()->Push(new Dialog("You cannot issue fleet movement orders while docked."));
+		else if(!player.HasVisited(*selectedSystem))
+			GetUI()->Push(new Dialog("You must visit this system before you can send your fleet there."));
+		else
+			player.SetEscortDestination(selectedSystem, uiClick / scale);
+	}
+	
+	return true;
+}
+
+
+
 // Draw the legend, correlating between a system's color and the value of the
 // selected "commodity," which may be reputation level, outfitter size, etc.
 void MapDetailPanel::DrawKey()
@@ -418,14 +459,15 @@ void MapDetailPanel::DrawInfo()
 	SpriteShader::Draw(systemSprite, uiPoint);
 	
 	const Font &font = FontSet::Get(14);
-	string systemName = player.KnowsName(selectedSystem) ?
+	string systemName = player.KnowsName(*selectedSystem) ?
 		selectedSystem->Name() : "Unexplored System";
-	font.Draw(systemName, uiPoint + Point(-90., -7.), medium);
+	const auto alignLeft = Layout(140, Truncate::BACK);
+	font.Draw({systemName, alignLeft}, uiPoint + Point(-90., -7.), medium);
 	
 	governmentY = uiPoint.Y() + 10.;
-	string gov = player.HasVisited(selectedSystem) ?
+	string gov = player.HasVisited(*selectedSystem) ?
 		selectedSystem->GetGovernment()->GetName() : "Unknown Government";
-	font.Draw(gov, uiPoint + Point(-90., 13.), (commodity == SHOW_GOVERNMENT) ? medium : dim);
+	font.Draw({gov, alignLeft}, uiPoint + Point(-90., 13.), (commodity == SHOW_GOVERNMENT) ? medium : dim);
 	if(commodity == SHOW_GOVERNMENT)
 		PointerShader::Draw(uiPoint + Point(-90., 20.), Point(1., 0.),
 			10.f, 10.f, 0.f, medium);
@@ -434,7 +476,7 @@ void MapDetailPanel::DrawInfo()
 	
 	planetY.clear();
 	// Draw the basic information for visitable planets in this system.
-	if(player.HasVisited(selectedSystem))
+	if(player.HasVisited(*selectedSystem))
 	{
 		set<const Planet *> shown;
 		const Sprite *planetSprite = SpriteSet::Get("ui/map planet");
@@ -450,8 +492,8 @@ void MapDetailPanel::DrawInfo()
 				
 				SpriteShader::Draw(planetSprite, uiPoint);
 				planetY[planet] = uiPoint.Y() - 60;
-			
-				font.Draw(object.Name(),
+				
+				font.Draw({object.Name(), alignLeft},
 					uiPoint + Point(-70., -52.),
 					planet == selectedPlanet ? medium : dim);
 				
@@ -481,7 +523,7 @@ void MapDetailPanel::DrawInfo()
 					PointerShader::Draw(uiPoint + Point(-60., 15.), Point(1., 0.),
 						10.f, 10.f, 0.f, medium);
 				
-				bool hasVisited = player.HasVisited(planet);
+				bool hasVisited = player.HasVisited(*planet);
 				font.Draw(hasVisited ? "(has been visited)" : "(not yet visited)",
 					uiPoint + Point(-70., 28.),
 					dim);
@@ -513,7 +555,7 @@ void MapDetailPanel::DrawInfo()
 		
 		string price;
 		
-		bool hasVisited = player.HasVisited(selectedSystem);
+		bool hasVisited = player.HasVisited(*selectedSystem);
 		if(hasVisited && selectedSystem->IsInhabited(player.Flagship()))
 		{
 			int value = selectedSystem->Trade(commodity.name);
@@ -538,8 +580,8 @@ void MapDetailPanel::DrawInfo()
 		else
 			price = (hasVisited ? "n/a" : "?");
 		
-		Point pos = uiPoint + Point(140. - font.Width(price), 0.);
-		font.Draw(price, pos, color);
+		const auto alignRight = Layout(140, Alignment::RIGHT, Truncate::BACK);
+		font.Draw({price, alignRight}, uiPoint, color);
 		
 		if(isSelected)
 			PointerShader::Draw(uiPoint + Point(0., 7.), Point(1., 0.), 10.f, 10.f, 0.f, color);
@@ -548,7 +590,7 @@ void MapDetailPanel::DrawInfo()
 	}
 	
 	if(selectedPlanet && !selectedPlanet->Description().empty()
-			&& player.HasVisited(selectedPlanet) && !selectedPlanet->IsWormhole())
+			&& player.HasVisited(*selectedPlanet) && !selectedPlanet->IsWormhole())
 	{
 		static const int X_OFFSET = 240;
 		static const int WIDTH = 500;
@@ -558,7 +600,7 @@ void MapDetailPanel::DrawInfo()
 		SpriteShader::Draw(panelSprite, pos);
 		
 		WrappedText text(font);
-		text.SetAlignment(WrappedText::JUSTIFIED);
+		text.SetAlignment(Alignment::JUSTIFIED);
 		text.SetWrapWidth(WIDTH - 20);
 		text.Wrap(selectedPlanet->Description());
 		text.Draw(Point(Screen::Right() - X_OFFSET - WIDTH, Screen::Top() + 20), medium);
@@ -576,7 +618,7 @@ void MapDetailPanel::DrawOrbits()
 	SpriteShader::Draw(orbitSprite, Screen::TopRight() + .5 * Point(-orbitSprite->Width(), orbitSprite->Height()));
 	Point orbitCenter = Screen::TopRight() + Point(-120., 160.);
 	
-	if(!selectedSystem || !player.HasVisited(selectedSystem))
+	if(!player.HasVisited(*selectedSystem))
 		return;
 	
 	const Font &font = FontSet::Get(14);
@@ -587,7 +629,7 @@ void MapDetailPanel::DrawOrbits()
 		maxDistance = max(maxDistance, object.Position().Length() + object.Radius());
 	
 	// 2400 -> 120.
-	double scale = .03;
+	scale = .03;
 	maxDistance *= scale;
 	
 	if(maxDistance > 115.)
@@ -641,6 +683,30 @@ void MapDetailPanel::DrawOrbits()
 		RingShader::Draw(pos, object.Radius() * scale + 1., 0.f, color);
 	}
 	
+	// If the player has a pending order for escorts to move to a new system, draw it.
+	if(player.HasEscortDestination())
+	{
+		auto pendingOrder = player.GetEscortDestination();
+		if(pendingOrder.first == selectedSystem)
+		{
+			// Draw an X (to mark the spot, of course).
+			auto uiPoint = (pendingOrder.second * scale) + orbitCenter;
+			const Color *color = GameData::Colors().Get("map orbits fleet destination");
+			// TODO: Add a "batch pointershader" method that takes the shape description, a count, and a reference point+orientation
+			// Use that method below and in Engine for drawing target reticles.
+			auto a = Angle{45.};
+			auto inc = Angle{90.};
+			
+			PointerShader::Bind();
+			for(int i = 0; i < 4; ++i)
+			{
+				PointerShader::Add(uiPoint, a.Unit(), 6.f, 6.f, -3.f, *color);
+				a += inc;
+			}
+			PointerShader::Unbind();
+		}
+	}
+	
 	// Draw the selection ring on top of everything else.
 	for(const StellarObject &object : selectedSystem->Objects())
 		if(selectedPlanet && object.GetPlanet() == selectedPlanet)
@@ -650,8 +716,9 @@ void MapDetailPanel::DrawOrbits()
 	
 	// Draw the name of the selected planet.
 	const string &name = selectedPlanet ? selectedPlanet->Name() : selectedSystem->Name();
-	Point namePos(Screen::Right() - .5 * font.Width(name) - 100., Screen::Top() + 7.);
-	font.Draw(name, namePos, *GameData::Colors().Get("medium"));
+	Point namePos(Screen::Right() - 190., Screen::Top() + 7.);
+	font.Draw({name, {180, Alignment::CENTER, Truncate::BACK}},
+		namePos, *GameData::Colors().Get("medium"));
 }
 
 
