@@ -1,4 +1,4 @@
-/* MissionAction.h
+/* MissionAction.cpp
 Copyright (c) 2014 by Michael Zahniser
 
 Endless Sky is free software: you can redistribute it and/or modify it under the
@@ -17,7 +17,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "DataNode.h"
 #include "DataWriter.h"
 #include "Dialog.h"
-#include "Format.h"
+#include "text/Format.h"
 #include "GameData.h"
 #include "GameEvent.h"
 #include "Messages.h"
@@ -28,11 +28,19 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "UI.h"
 
 #include <cstdlib>
-#include <vector>
 
 using namespace std;
 
 namespace {
+	void DoGift(PlayerInfo &player, const Ship *model, const string &name)
+	{
+		if(model->ModelName().empty())
+			return;
+		
+		player.BuyShip(model, name, true);
+		Messages::Add("The " + model->ModelName() + " \"" + name + "\" was added to your fleet.");
+	}
+	
 	void DoGift(PlayerInfo &player, const Outfit *outfit, int count, UI *ui)
 	{
 		Ship *flagship = player.Flagship();
@@ -186,11 +194,18 @@ void MissionAction::Load(const DataNode &node, const string &missionName)
 			conversation.Load(child);
 		else if(key == "conversation" && hasValue)
 			stockConversation = GameData::Conversations().Get(child.Token(1));
+		else if(key == "give" && hasValue)
+		{
+			if(child.Token(1) == "ship" && child.Size() >= 3)
+				giftShips.emplace_back(GameData::Ships().Get(child.Token(2)), child.Size() >= 4 ? child.Token(3) : "");
+			else
+				child.PrintTrace("Skipping unsupported \"give\" syntax:");
+		}
 		else if(key == "outfit" && hasValue)
 		{
 			int count = (child.Size() < 3 ? 1 : static_cast<int>(child.Value(2)));
 			if(count)
-				gifts[GameData::Outfits().Get(child.Token(1))] = count;
+				giftOutfits[GameData::Outfits().Get(child.Token(1))] = count;
 			else
 			{
 				// outfit <outfit> 0 means the player must have this outfit.
@@ -297,7 +312,9 @@ void MissionAction::Save(DataWriter &out) const
 		if(!conversation.IsEmpty())
 			conversation.Save(out);
 		
-		for(const auto &it : gifts)
+		for(const auto &it : giftShips)
+			out.Write("give", "ship", it.first->VariantName(), it.second);
+		for(const auto &it : giftOutfits)
 			out.Write("outfit", it.first->Name(), it.second);
 		for(const auto &it : requiredOutfits)
 			out.Write("require", it.first->Name(), it.second);
@@ -310,8 +327,8 @@ void MissionAction::Save(DataWriter &out) const
 			else
 				out.Write("event", it.first->Name(), it.second.first, it.second.second);
 		}
-		for(const auto &name : fail)
-			out.Write("fail", name);
+		for(auto &&missionName : fail)
+			out.Write("fail", missionName);
 		
 		conditions.Save(out);
 	}
@@ -320,6 +337,46 @@ void MissionAction::Save(DataWriter &out) const
 
 
 
+// Check this template or instantiated MissionAction to see if any used content
+// is not fully defined (e.g. plugin removal, typos in names, etc.).
+bool MissionAction::IsValid() const
+{
+	// Any filter used to control where this action triggers must be valid.
+	if(!systemFilter.IsValid())
+		return false;
+	
+	// Stock phrases that generate text must be defined.
+	if(stockDialogPhrase && stockDialogPhrase->Name().empty())
+		return false;
+	
+	// Stock conversations must be defined.
+	if(stockConversation && stockConversation->IsEmpty())
+		return false;
+	
+	// Events which get activated by this action must be valid.
+	for(auto &&event : events)
+		if(!event.first->IsValid())
+			return false;
+
+	// Gifted or required content must be defined & valid.
+	for(auto &&it : giftShips)
+		if(!it.first->IsValid())
+			return false;
+	for(auto &&outfit : giftOutfits)
+		if(!outfit.first->IsDefined())
+			return false;
+	for(auto &&outfit : requiredOutfits)
+		if(!outfit.first->IsDefined())
+			return false;
+	
+	// It is OK for this action to try to fail a mission that does not exist.
+	// (E.g. a plugin may be designed for interoperability with other plugins.)
+	
+	return true;
+}
+
+
+
 int MissionAction::Payment() const
 {
 	return payment;
@@ -327,6 +384,13 @@ int MissionAction::Payment() const
 
 
 
+const string &MissionAction::DialogText() const
+{
+	return dialogText;
+}
+
+
+
 // Check if this action can be completed right now. It cannot be completed
 // if it takes away money or outfits that the player does not have.
 bool MissionAction::CanBeDone(const PlayerInfo &player, const shared_ptr<Ship> &boardingShip) const
@@ -335,7 +399,7 @@ bool MissionAction::CanBeDone(const PlayerInfo &player, const shared_ptr<Ship> &
 		return false;
 	
 	const Ship *flagship = player.Flagship();
-	for(const auto &it : gifts)
+	for(const auto &it : giftOutfits)
 	{
 		// If this outfit is being given, the player doesn't need to have it.
 		if(it.second > 0)
@@ -394,7 +458,7 @@ bool MissionAction::CanBeDone(const PlayerInfo &player, const shared_ptr<Ship> &
 
 
 
-void MissionAction::Do(PlayerInfo &player, UI *ui, const System *destination, const shared_ptr<Ship> &ship) const
+void MissionAction::Do(PlayerInfo &player, UI *ui, const System *destination, const shared_ptr<Ship> &ship, const bool isUnique) const
 {
 	bool isOffer = (trigger == "offer");
 	if(!conversation.IsEmpty() && ui)
@@ -419,9 +483,14 @@ void MissionAction::Do(PlayerInfo &player, UI *ui, const System *destination, co
 			subs["<ship>"] = player.Flagship()->Name();
 		string text = Format::Replace(dialogText, subs);
 		
+		// Don't push the dialog text if this is a visit action on a nonunique
+		// mission; on visit, nonunique dialogs are handled by PlayerInfo as to
+		// avoid the player being spammed by dialogs if they have multiple
+		// missions active with the same destination (e.g. in the case of
+		// stacking bounty jobs).
 		if(isOffer)
 			ui->Push(new Dialog(text, player, destination));
-		else
+		else if(isUnique || trigger != "visit")
 			ui->Push(new Dialog(text));
 	}
 	else if(isOffer && ui)
@@ -433,12 +502,14 @@ void MissionAction::Do(PlayerInfo &player, UI *ui, const System *destination, co
 		for(const auto &eit : it.second)
 			player.AddSpecialLog(it.first, eit.first, eit.second);
 	
+	for(const auto &it : giftShips)
+		DoGift(player, it.first, it.second);
 	// If multiple outfits are being transferred, first remove them before
 	// adding any new ones.
-	for(const auto &it : gifts)
+	for(const auto &it : giftOutfits)
 		if(it.second < 0)
 			DoGift(player, it.first, it.second, ui);
-	for(const auto &it : gifts)
+	for(const auto &it : giftOutfits)
 		if(it.second > 0)
 			DoGift(player, it.first, it.second, ui);
 	
@@ -466,7 +537,8 @@ void MissionAction::Do(PlayerInfo &player, UI *ui, const System *destination, co
 
 
 
-MissionAction MissionAction::Instantiate(map<string, string> &subs, const System *origin, int jumps, int payload) const
+// Convert this validated template into a populated action.
+MissionAction MissionAction::Instantiate(map<string, string> &subs, const System *origin, int jumps, int64_t payload) const
 {
 	MissionAction result;
 	result.trigger = trigger;
@@ -474,15 +546,17 @@ MissionAction MissionAction::Instantiate(map<string, string> &subs, const System
 	// Convert any "distance" specifiers into "near <system>" specifiers.
 	result.systemFilter = systemFilter.SetOrigin(origin);
 	
+	// All contained events are valid, else we would not be calling Instantiate. For these
+	// valid events, pick a date within the specified range on which the event will occur.
 	for(const auto &it : events)
 	{
-		// Allow randomization of event times. The second value in the pair is
-		// always greater than or equal to the first, so Random::Int() will
-		// never be called with a value less than 1.
 		int day = it.second.first + Random::Int(it.second.second - it.second.first + 1);
 		result.events[it.first] = make_pair(day, day);
 	}
-	result.gifts = gifts;
+	for(const auto &it : giftShips)
+		result.giftShips.emplace_back(it.first, !it.second.empty() ? it.second : GameData::Phrases().Get("civilian")->Get());
+	result.giftOutfits = giftOutfits;
+	result.requiredOutfits = requiredOutfits;
 	result.payment = payment + (jumps + 1) * payload * paymentMultiplier;
 	// Fill in the payment amount if this is the "complete" action.
 	string previousPayment = subs["<payment>"];
