@@ -23,8 +23,11 @@ using namespace std;
 namespace {
 	Shader shader;
 	GLint scaleI;
+	GLint offI;
 	GLint transformI;
 	GLint positionI;
+	GLint frameI;
+	GLint frameCountI;
 	GLint colorI;
 	
 	GLuint vao;
@@ -36,56 +39,87 @@ namespace {
 void OutlineShader::Init()
 {
 	static const char *vertexCode =
-		"uniform mat2 transform;\n"
-		"uniform vec2 position;\n"
+		"// vertex outline shader\n"
 		"uniform vec2 scale;\n"
+		"uniform vec2 position;\n"
+		"uniform mat2 transform;\n"
+		
 		"in vec2 vert;\n"
 		"in vec2 vertTexCoord;\n"
-		"out vec2 tc;\n"
-		"out vec2 off;\n"
+		
+		"out vec2 fragTexCoord;\n"
+		
 		"void main() {\n"
-		"  tc = vertTexCoord;\n"
-		"  mat2 sq = matrixCompMult(transform, transform);\n"
-		"  off = vec2(.5 / sqrt(sq[0][0] + sq[0][1]), .5 / sqrt(sq[1][0] + sq[1][1]));\n"
+		"  fragTexCoord = vertTexCoord;\n"
 		"  gl_Position = vec4((transform * vert + position) * scale, 0, 1);\n"
 		"}\n";
-
+	
+	// The outline shader applies a Sobel filter to an image. The alpha channel
+	// (i.e. the silhouette of the sprite) is given the most weight, but some
+	// weight is also given to the RGB values so that there will be some detail
+	// in the interior of the silhouette as well.
+	
+	// To reduce sampling error and bring out fine details, for every output
+	// pixel the Sobel filter is actually applied in a 3x3 neighborhood and
+	// averaged together. That neighborhood's scale is .618034 times the scale
+	// of the Sobel neighborhood (i.e. the golden ratio) to minimize any
+	// aliasing effects between the two.
 	static const char *fragmentCode =
-		"uniform sampler2D tex;\n"
+		"// fragment outline shader\n"
+		"uniform sampler2DArray tex;\n"
+		"uniform float frame = 0;\n"
+		"uniform float frameCount = 0;\n"
 		"uniform vec4 color = vec4(1, 1, 1, 1);\n"
-		"in vec2 tc;\n"
-		"in vec2 off;\n"
+		"uniform vec2 off;\n"
+		"const vec4 weight = vec4(.4, .4, .4, 1.);\n"
+		
+		"in vec2 fragTexCoord;\n"
+		
 		"out vec4 finalColor;\n"
-		"void main() {\n"
+		
+		"float Sobel(float layer) {\n"
 		"  float sum = 0;\n"
 		"  for(int dy = -1; dy <= 1; ++dy)\n"
 		"  {\n"
 		"    for(int dx = -1; dx <= 1; ++dx)\n"
 		"    {\n"
-		"      vec2 d = vec2(.618 * dx * off.x, .618 * dy * off.y);\n"
-		"      float ae = texture(tex, d + vec2(tc.x - off.x, tc.y)).a;\n"
-		"      float aw = texture(tex, d + vec2(tc.x + off.x, tc.y)).a;\n"
-		"      float an = texture(tex, d + vec2(tc.x, tc.y - off.y)).a;\n"
-		"      float as = texture(tex, d + vec2(tc.x, tc.y + off.y)).a;\n"
-		"      float ane = texture(tex, d + vec2(tc.x - off.x, tc.y - off.y)).a;\n"
-		"      float anw = texture(tex, d + vec2(tc.x + off.x, tc.y - off.y)).a;\n"
-		"      float ase = texture(tex, d + vec2(tc.x - off.x, tc.y + off.y)).a;\n"
-		"      float asw = texture(tex, d + vec2(tc.x + off.x, tc.y + off.y)).a;\n"
-		"      float h = (ae * 2 + ane + ase) - (aw * 2 + anw + asw);\n"
-		"      float v = (an * 2 + ane + anw) - (as * 2 + ase + asw);\n"
+		"      vec2 center = fragTexCoord + .618034 * off * vec2(dx, dy);\n"
+		"      float nw = dot(texture(tex, vec3(center + vec2(-off.x, -off.y), layer)), weight);\n"
+		"      float ne = dot(texture(tex, vec3(center + vec2(off.x, -off.y), layer)), weight);\n"
+		"      float sw = dot(texture(tex, vec3(center + vec2(-off.x, off.y), layer)), weight);\n"
+		"      float se = dot(texture(tex, vec3(center + vec2(off.x, off.y), layer)), weight);\n"
+		"      float h = nw + sw - ne - se + 2 * (\n"
+		"        dot(texture(tex, vec3(center + vec2(-off.x, 0), layer)), weight)\n"
+		"          - dot(texture(tex, vec3(center + vec2(off.x, 0), layer)), weight));\n"
+		"      float v = nw + ne - sw - se + 2 * (\n"
+		"        dot(texture(tex, vec3(center + vec2(0, -off.y), layer)), weight)\n"
+		"          - dot(texture(tex, vec3(center + vec2(0, off.y), layer)), weight));\n"
 		"      sum += h * h + v * v;\n"
 		"    }\n"
 		"  }\n"
-		"  finalColor = color * sqrt(sum / 144);\n"
+		"  return sum;\n"
+		"}\n"
+		
+		"void main() {\n"
+		"  float first = floor(frame);\n"
+		"  float second = mod(ceil(frame), frameCount);\n"
+		"  float fade = frame - first;\n"
+		"  float sum = mix(Sobel(first), Sobel(second), fade);\n"
+		"  finalColor = color * sqrt(sum / 180);\n"
 		"}\n";
 	
 	shader = Shader(vertexCode, fragmentCode);
 	scaleI = shader.Uniform("scale");
+	offI = shader.Uniform("off");
 	transformI = shader.Uniform("transform");
 	positionI = shader.Uniform("position");
+	frameI = shader.Uniform("frame");
+	frameCountI = shader.Uniform("frameCount");
 	colorI = shader.Uniform("color");
 	
-	glUniform1ui(shader.Uniform("tex"), 0);
+	glUseProgram(shader.Object());
+	glUniform1i(shader.Uniform("tex"), 0);
+	glUseProgram(0);
 	
 	// Generate the vertex data for drawing sprites.
 	glGenVertexArrays(1, &vao);
@@ -100,14 +134,15 @@ void OutlineShader::Init()
 		-.5f,  .5f, 0.f, 1.f,
 		 .5f,  .5f, 1.f, 1.f
 	};
+	constexpr auto stride = 4 * sizeof(GLfloat);
 	glBufferData(GL_ARRAY_BUFFER, sizeof(vertexData), vertexData, GL_STATIC_DRAW);
 	
 	glEnableVertexAttribArray(shader.Attrib("vert"));
-	glVertexAttribPointer(shader.Attrib("vert"), 2, GL_FLOAT, GL_FALSE, 4 * sizeof(GLfloat), nullptr);
+	glVertexAttribPointer(shader.Attrib("vert"), 2, GL_FLOAT, GL_FALSE, stride, nullptr);
 	
 	glEnableVertexAttribArray(shader.Attrib("vertTexCoord"));
 	glVertexAttribPointer(shader.Attrib("vertTexCoord"), 2, GL_FLOAT, GL_TRUE,
-		4 * sizeof(GLfloat), (const GLvoid*)(2 * sizeof(GLfloat)));
+		stride, reinterpret_cast<const GLvoid*>(2 * sizeof(GLfloat)));
 	
 	// unbind the VBO and VAO
 	glBindBuffer(GL_ARRAY_BUFFER, 0);
@@ -116,15 +151,22 @@ void OutlineShader::Init()
 
 
 
-void OutlineShader::Draw(const Sprite *sprite, const Point &pos, const Point &size, const Color &color, const Point &unit, int frame)
+void OutlineShader::Draw(const Sprite *sprite, const Point &pos, const Point &size, const Color &color, const Point &unit, float frame)
 {
 	glUseProgram(shader.Object());
 	glBindVertexArray(vao);
-	glActiveTexture(GL_TEXTURE0);
 	
 	GLfloat scale[2] = {2.f / Screen::Width(), -2.f / Screen::Height()};
 	glUniform2fv(scaleI, 1, scale);
 	
+	GLfloat off[2] = {
+		static_cast<float>(.5 / size.X()),
+		static_cast<float>(.5 / size.Y())};
+	glUniform2fv(offI, 1, off);
+	
+	glUniform1f(frameI, frame);
+	glUniform1f(frameCountI, sprite->Frames());
+	
 	Point uw = unit * size.X();
 	Point uh = unit * size.Y();
 	GLfloat transform[4] = {
@@ -141,7 +183,7 @@ void OutlineShader::Draw(const Sprite *sprite, const Point &pos, const Point &si
 	
 	glUniform4fv(colorI, 1, color.Get());
 	
-	glBindTexture(GL_TEXTURE_2D, sprite->Texture(frame));
+	glBindTexture(GL_TEXTURE_2D_ARRAY, sprite->Texture(unit.Length() * Screen::Zoom() > 50.));
 	
 	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
 	
