@@ -15,12 +15,14 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include "Account.h"
 #include "CargoHold.h"
+#include "CoreStartData.h"
+#include "DataNode.h"
 #include "Date.h"
 #include "Depreciation.h"
 #include "GameEvent.h"
 #include "Mission.h"
-#include "Planet.h"
 
+#include <chrono>
 #include <list>
 #include <map>
 #include <memory>
@@ -29,14 +31,13 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include <utility>
 #include <vector>
 
-class DataNode;
 class Government;
 class Outfit;
-class Person;
 class Planet;
 class Rectangle;
 class Ship;
 class ShipEvent;
+class StartConditions;
 class StellarObject;
 class System;
 class UI;
@@ -59,11 +60,11 @@ public:
 	// Check if any player's information is loaded.
 	bool IsLoaded() const;
 	// Make a new player.
-	void New();
+	void New(const StartConditions &start);
 	// Load an existing player.
 	void Load(const std::string &path);
-	// Load the most recently saved player.
-	void LoadRecent();
+	// Load the most recently saved player. If no save could be loaded, returns false.
+	bool LoadRecent();
 	// Save this player (using the Identifier() as the file name).
 	void Save() const;
 	
@@ -77,7 +78,7 @@ public:
 	void AddEvent(const GameEvent &event, const Date &date);
 	
 	// Mark the player as dead, or check if they have died.
-	void Die(bool allShipsDie = false);
+	void Die(int response = 0, const std::shared_ptr<Ship> &capturer = nullptr);
 	bool IsDead() const;
 	
 	// Get or set the player's name.
@@ -89,12 +90,15 @@ public:
 	const Date &GetDate() const;
 	void IncrementDate();
 	
+	// Get basic data about the player's starting scenario.
+	const CoreStartData &StartData() const noexcept;
+	
 	// Set the system the player is in. This must be stored here so that even if
 	// the player sells all their ships, we still know where the player is.
 	// This also marks the given system as visited.
-	void SetSystem(const System *system);
+	void SetSystem(const System &system);
 	const System *GetSystem() const;
-	// Set what planet the player is on.
+	// Set what planet the player is on (or nullptr, if taking off).
 	void SetPlanet(const Planet *planet);
 	const Planet *GetPlanet() const;
 	// If the player is landed, return the stellar object they are on.
@@ -103,23 +107,28 @@ public:
 	// must leave the planet immediately (without time to do anything else).
 	bool ShouldLaunch() const;
 	
-	// Access the player's accountign information.
+	// Access the player's accounting information.
 	const Account &Accounts() const;
 	Account &Accounts();
 	// Calculate the daily salaries for crew, not counting crew on "parked" ships.
 	int64_t Salaries() const;
+	// Calculate the daily maintenance cost for all ships and in cargo outfits.
+	int64_t Maintenance() const;
 	
 	// Access the flagship (the first ship in the list). This returns null if
-	// the player does not have any ships.
+	// the player does not have any ships that can be a flagship.
 	const Ship *Flagship() const;
 	Ship *Flagship();
 	const std::shared_ptr<Ship> &FlagshipPtr();
 	// Get the full list of ships the player owns.
 	const std::vector<std::shared_ptr<Ship>> &Ships() const;
+	// Inspect the flightworthiness of the player's active fleet as a whole to
+	// determine which ships cannot travel with the group.
+	std::map<const std::shared_ptr<Ship>, std::vector<std::string>> FlightCheck() const;
 	// Add a captured ship to your fleet.
 	void AddShip(const std::shared_ptr<Ship> &ship);
 	// Buy or sell a ship.
-	void BuyShip(const Ship *model, const std::string &name);
+	void BuyShip(const Ship *model, const std::string &name, bool isGift = false);
 	void SellShip(const Ship *selected);
 	void DisownShip(const Ship *selected);
 	void ParkShip(const Ship *selected, bool isParked);
@@ -127,10 +136,16 @@ public:
 	// Change the order of the given ship in the list.
 	void ReorderShip(int fromIndex, int toIndex);
 	int ReorderShips(const std::set<int> &fromIndices, int toIndex);
+	// Get the attraction factors of the player's fleet to raid fleets.
+	std::pair<double, double> RaidFleetFactors() const;
 	
 	// Get cargo information.
 	CargoHold &Cargo();
 	const CargoHold &Cargo() const;
+	// Get items stored on the player's current planet.
+	CargoHold *Storage(bool forceCreate = false);
+	// Get items stored on all planets (for map display).
+	const std::map<const Planet *, CargoHold> &PlanetaryStorage() const;
 	// Get cost basis for commodities.
 	void AdjustBasis(const std::string &commodity, int64_t adjustment);
 	int64_t GetBasis(const std::string &commodity, int tons = 1) const;
@@ -140,6 +155,10 @@ public:
 	void Land(UI *ui);
 	// Load the cargo back into your ships. This may require selling excess.
 	bool TakeOff(UI *ui);
+
+	// Get or add to pilot's playtime.
+	double GetPlayTime() const noexcept;
+	void AddPlayTime(std::chrono::nanoseconds timeVal);
 	
 	// Get the player's logbook.
 	const std::multimap<Date, std::string> &Logbook() const;
@@ -151,17 +170,21 @@ public:
 	// Get mission information.
 	const std::list<Mission> &Missions() const;
 	const std::list<Mission> &AvailableJobs() const;
+	const Mission *ActiveBoardingMission() const;
+	void UpdateMissionNPCs();
 	void AcceptJob(const Mission &mission, UI *ui);
-	// Check to see if there is any mission to offer in the spaceport right now.
+	// Check to see if there is any mission to offer right now.
 	Mission *MissionToOffer(Mission::Location location);
 	Mission *BoardingMission(const std::shared_ptr<Ship> &ship);
-	const std::shared_ptr<Ship> &BoardingShip() const;
+	void ClearActiveBoardingMission();
 	// If one of your missions cannot be offered because you do not have enough
 	// space for it, and it specifies a message to be shown in that situation,
 	// show that message.
 	void HandleBlockedMissions(Mission::Location location, UI *ui);
 	// Callback for accepting or declining whatever mission has been offered.
 	void MissionCallback(int response);
+	// Basic callback for handling forced departure from a planet.
+	void BasicCallback(int response);
 	// Complete or fail a mission.
 	void RemoveMission(Mission::Trigger trigger, const Mission &mission, UI *ui);
 	// Mark a mission as failed, but do not remove it from the mission list yet.
@@ -170,24 +193,25 @@ public:
 	void HandleEvent(const ShipEvent &event, UI *ui);
 	
 	// Access the "condition" flags for this player.
-	int GetCondition(const std::string &name) const;
-	std::map<std::string, int> &Conditions();
-	const std::map<std::string, int> &Conditions() const;
-	// Set and check the reputation conditions, which missions can use to modify
-	// the player's reputation.
+	int64_t GetCondition(const std::string &name) const;
+	std::map<std::string, int64_t> &Conditions();
+	const std::map<std::string, int64_t> &Conditions() const;
+	// Set and check the reputation conditions, which missions and events
+	// can use to modify the player's reputation with other governments.
 	void SetReputationConditions();
 	void CheckReputationConditions();
 	
 	// Check what the player knows about the given system or planet.
-	bool HasSeen(const System *system) const;
-	bool HasVisited(const System *system) const;
-	bool HasVisited(const Planet *planet) const;
-	bool KnowsName(const System *system) const;
-	void Visit(const System *system);
-	void Visit(const Planet *planet);
+	bool HasSeen(const System &system) const;
+	bool HasVisited(const System &system) const;
+	bool HasVisited(const Planet &planet) const;
+	bool KnowsName(const System &system) const;
+	// Marking a system as visited also "sees" its neighbors.
+	void Visit(const System &system);
+	void Visit(const Planet &planet);
 	// Mark a system and its planets as unvisited, even if visited previously.
-	void Unvisit(const System *system);
-	void Unvisit(const Planet *planet);
+	void Unvisit(const System &system);
+	void Unvisit(const Planet &planet);
 	
 	// Access the player's travel plan.
 	bool HasTravelPlan() const;
@@ -226,6 +250,11 @@ public:
 	void Harvest(const Outfit *type);
 	const std::set<std::pair<const System *, const Outfit *>> &Harvested() const;
 	
+	// Get or set the travel destination for selected escorts via the map.
+	const std::pair<const System *, Point> &GetEscortDestination() const;
+	void SetEscortDestination(const System *system = nullptr, Point pos = Point());
+	bool HasEscortDestination() const;
+	
 	// Get or set what coloring is currently selected in the map.
 	int MapColoring() const;
 	void SetMapColoring(int index);
@@ -237,23 +266,32 @@ public:
 	
 	
 private:
-	// Don't anyone else to copy this class, because pointers won't get
+	// Don't allow anyone else to copy this class, because pointers won't get
 	// transferred properly.
 	PlayerInfo(const PlayerInfo &) = default;
 	PlayerInfo &operator=(const PlayerInfo &) = default;
 	
 	// Apply any "changes" saved in this player info to the global game state.
 	void ApplyChanges();
+	// After loading & applying changes, make sure the player & ship locations are sensible.
+	void ValidateLoad();
 	
 	// New missions are generated each time you land on a planet.
-	void UpdateAutoConditions();
+	void UpdateAutoConditions(bool isBoarding = false);
 	void CreateMissions();
+	void StepMissions(UI *ui);
 	void Autosave() const;
 	void Save(const std::string &path) const;
 	
+	// Check for and apply any punitive actions from planetary security.
+	void Fine(UI *ui);
+	
 	// Helper function to update the ship selection.
 	void SelectShip(const std::shared_ptr<Ship> &ship, bool *first);
 	
+	// Check that this player's current state can be saved.
+	bool CanBeSaved() const;
+	
 	
 private:
 	std::string firstName;
@@ -264,9 +302,11 @@ private:
 	const System *system = nullptr;
 	const Planet *planet = nullptr;
 	bool shouldLaunch = false;
-	bool hasFullClearance = true;
 	bool isDead = false;
 	
+	// The amount of in-game time played, in seconds.
+	double playTime = 0.0;
+	
 	Account accounts;
 	
 	std::shared_ptr<Ship> flagship;
@@ -274,21 +314,30 @@ private:
 	std::vector<std::weak_ptr<Ship>> selectedShips;
 	std::map<const Ship *, int> groups;
 	CargoHold cargo;
+	std::map<const Planet *, CargoHold> planetaryStorage;
 	std::map<std::string, int64_t> costBasis;
 	
 	std::multimap<Date, std::string> logbook;
 	std::map<std::string, std::map<std::string, std::string>> specialLogs;
 	
+	// A list of the player's active, accepted missions.
 	std::list<Mission> missions;
 	// These lists are populated when you land on a planet, and saved so that
 	// they will not change if you reload the game.
 	std::list<Mission> availableJobs;
 	std::list<Mission> availableMissions;
-	std::list<Mission> boardingMissions;
-	std::shared_ptr<Ship> boardingShip;
+	// If any mission component is not fully defined, the mission is deactivated
+	// until its components are fully evaluable (i.e. needed plugins are reinstalled).
+	std::list<Mission> inactiveMissions;
+	// Missions that are failed or aborted, but not yet deleted, and any
+	// missions offered while in-flight are not saved.
 	std::list<Mission> doneMissions;
+	std::list<Mission> boardingMissions;
+	// This pointer to the most recently accepted boarding mission enables
+	// its NPCs to be placed before the player lands, and is then cleared.
+	Mission *activeBoardingMission = nullptr;
 	
-	std::map<std::string, int> conditions;
+	std::map<std::string, int64_t> conditions;
 	
 	std::set<const System *> seen;
 	std::set<const System *> visitedSystems;
@@ -308,17 +357,24 @@ private:
 	std::list<DataNode> dataChanges;
 	DataNode economy;
 	// Persons that have been killed in this player's universe:
-	std::list<const Person *> destroyedPersons;
+	std::vector<std::string> destroyedPersons;
 	// Events that are going to happen some time in the future:
 	std::list<GameEvent> gameEvents;
 	
+	// The system and position therein to which the "orbits" system UI issued a move order.
+	std::pair<const System *, Point> interstellarEscortDestination;
 	// Currently selected coloring, in the map panel (defaults to reputation):
 	int mapColoring = -6;
 	int mapZoom = 0;
+	
 	// Currently collapsed categories for various panels.
 	std::map<std::string, std::set<std::string>> collapsed;
 	
 	bool freshlyLoaded = true;
+	int desiredCrew = 0;
+	
+	// Basic information about the player's starting scenario.
+	CoreStartData startData;
 };
 
 
