@@ -17,6 +17,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "DataNode.h"
 #include "DataWriter.h"
 #include "Files.h"
+#include "GameWindow.h"
 #include "Screen.h"
 
 #include <algorithm>
@@ -29,11 +30,16 @@ namespace {
 	int scrollSpeed = 60;
 	
 	// Strings for ammo expenditure:
-	static const string EXPEND_AMMO = "Escorts expend ammo";
-	static const string FRUGAL_ESCORTS = "Escorts use ammo frugally";
+	const string EXPEND_AMMO = "Escorts expend ammo";
+	const string FRUGAL_ESCORTS = "Escorts use ammo frugally";
 	
-	static const vector<double> ZOOMS = {.25, .35, .50, .70, 1.00, 1.40, 2.00};
+	const vector<double> ZOOMS = {.25, .35, .50, .70, 1.00, 1.40, 2.00};
 	int zoomIndex = 4;
+	constexpr double VOLUME_SCALE = .25;
+	
+	// Enable standard VSync by default.
+	const vector<string> VSYNC_SETTINGS = {"off", "on", "adaptive"};
+	int vsyncIndex = 1;
 }
 
 
@@ -44,15 +50,20 @@ void Preferences::Load()
 	// values for settings that are off by default.
 	settings["Automatic aiming"] = true;
 	settings["Render motion blur"] = true;
-	settings["Escorts use ammo frugally"] = true;
-	settings["Escorts expend ammo"] = true;
+	settings[FRUGAL_ESCORTS] = true;
+	settings[EXPEND_AMMO] = true;
+	settings["Damaged fighters retreat"] = true;
 	settings["Warning siren"] = true;
+	settings["Show escort systems on map"] = true;
 	settings["Show mini-map"] = true;
 	settings["Show planet labels"] = true;
 	settings["Show hyperspace flash"] = true;
 	settings["Draw background haze"] = true;
+	settings["Draw starfield"] = true;
 	settings["Hide unexplored map regions"] = true;
 	settings["Turrets focus fire"] = true;
+	settings["Ship outlines in shops"] = true;
+	settings["Interrupt fast-forward"] = true;
 	
 	DataFile prefs(Files::Config() + "preferences.txt");
 	for(const DataNode &node : prefs)
@@ -62,11 +73,13 @@ void Preferences::Load()
 		else if(node.Token(0) == "zoom" && node.Size() >= 2)
 			Screen::SetZoom(node.Value(1));
 		else if(node.Token(0) == "volume" && node.Size() >= 2)
-			Audio::SetVolume(node.Value(1));
+			Audio::SetVolume(node.Value(1) * VOLUME_SCALE);
 		else if(node.Token(0) == "scroll speed" && node.Size() >= 2)
 			scrollSpeed = node.Value(1);
 		else if(node.Token(0) == "view zoom")
-			zoomIndex = node.Value(1);
+			zoomIndex = max<int>(0, min<int>(node.Value(1), ZOOMS.size() - 1));
+		else if(node.Token(0) == "vsync")
+			vsyncIndex = max<int>(0, min<int>(node.Value(1), VSYNC_SETTINGS.size() - 1));
 		else
 			settings[node.Token(0)] = (node.Size() == 1 || node.Value(1));
 	}
@@ -78,11 +91,12 @@ void Preferences::Save()
 {
 	DataWriter out(Files::Config() + "preferences.txt");
 	
-	out.Write("volume", Audio::Volume());
+	out.Write("volume", Audio::Volume() / VOLUME_SCALE);
 	out.Write("window size", Screen::RawWidth(), Screen::RawHeight());
-	out.Write("zoom", Screen::Zoom());
+	out.Write("zoom", Screen::UserZoom());
 	out.Write("scroll speed", scrollSpeed);
 	out.Write("view zoom", zoomIndex);
+	out.Write("vsync", vsyncIndex);
 	
 	for(const auto &it : settings)
 		out.Write(it.first, it.second);
@@ -120,6 +134,8 @@ string Preferences::AmmoUsage()
 	return Has(EXPEND_AMMO) ? Has(FRUGAL_ESCORTS) ? "frugally" : "always" : "never";
 }
 
+
+
 // Scroll speed preference.
 int Preferences::ScrollSpeed()
 {
@@ -162,3 +178,43 @@ bool Preferences::ZoomViewOut()
 	--zoomIndex;
 	return true;
 }
+
+
+
+bool Preferences::ToggleVSync()
+{
+	int targetIndex = vsyncIndex + 1;
+	if(targetIndex == static_cast<int>(VSYNC_SETTINGS.size()))
+		targetIndex = 0;
+	if(!GameWindow::SetVSync(static_cast<VSync>(targetIndex)))
+	{
+		// Not all drivers support adaptive VSync. Increment desired VSync again.
+		++targetIndex;
+		if(targetIndex == static_cast<int>(VSYNC_SETTINGS.size()))
+			targetIndex = 0;
+		if(!GameWindow::SetVSync(static_cast<VSync>(targetIndex)))
+		{
+			// Restore original saved setting.
+			Files::LogError("Unable to change VSync state");
+			GameWindow::SetVSync(static_cast<VSync>(vsyncIndex));
+			return false;
+		}
+	}
+	vsyncIndex = targetIndex;
+	return true;
+}
+
+
+
+// Return the current VSync setting
+Preferences::VSync Preferences::VSyncState()
+{
+	return static_cast<VSync>(vsyncIndex);
+}
+
+
+
+const string &Preferences::VSyncSetting()
+{
+	return VSYNC_SETTINGS[vsyncIndex];
+}
