@@ -14,9 +14,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #define STELLAR_OBJECT_H_
 
 #include "Body.h"
-#include "Point.h"
 
-class Color;
 class Planet;
 class Ship;
 
